<?php

namespace Drupal\Tests\bakery\Unit\EventSubscriber;

use Drupal\bakery\EventSubscriber\CookieMonster;
use Drupal\Component\Datetime\Time;
use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\MemoryStorage;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Http\RequestStack;
use Drupal\Tests\UnitTestCase;
use Symfony\Component\EventDispatcher\EventDispatcher;
use Symfony\Component\HttpFoundation\Cookie;
use Symfony\Component\HttpFoundation\ParameterBag;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * @coversDefaultClass \Drupal\bakery\EventSubscriber\CookieMonster
 */
class CookieMonsterTest extends UnitTestCase {

  /**
   * @var \Drupal\bakery\EventSubscriber\CookieMonster
   */
  private $sot;

  /**
   * @var \Symfony\Component\HttpFoundation\ParameterBag
   */
  private $cookieJar;

  /**
   * @var \Drupal\Core\Http\RequestStack
   */
  private $request;

  /**
   * @var \Drupal\Component\Datetime\Time
   */
  private $time;

  public function setUp() {
    parent::setUp();
    $this->request = new RequestStack();
    $this->request->push(new Request());
    $this->time = new Time($this->request);

    $config = new Config(
      'config.test',
      new MemoryStorage(),
      new EventDispatcher(),
      $this->prophesize(TypedConfigManagerInterface::class)->reveal()
    );
    $config->set('bakery_domain', '.example.com');
    $config->set('bakery_freshness', 3600);
    $cf = $this->prophesize(ConfigFactoryInterface::class);
    $cf->get('bakery.settings')->willReturn($config);
    $this->cookieJar = new ParameterBag();

    $this->sot = new CookieMonster($this->time, $cf->reveal(), $this->cookieJar);
  }

  /**
   * @covers ::meWantCookie
   */
  public function testFoo() {
    $response = new Response();
    $event = new ResponseEvent(
      $this->prophesize(HttpKernelInterface::class)->reveal(),
      $this->request->getCurrentRequest(),
      HttpKernelInterface::MASTER_REQUEST,
      $response
    );

    $this->sot->meWantCookie($event);
    $this->assertCount(0, $response->headers->getCookies());

    $this->cookieJar->set('cookie1', 'foo');
    $this->cookieJar->set('cookie2', 'bar');
    $this->cookieJar->set('cookie3', '');
    $this->sot->meWantCookie($event);
    $this->assertCount(4, $response->headers->getCookies());
    $expire = $this->time->getRequestTime() + 3600;
    $secure = !empty(ini_get('session.cookie_secure'));
    $this->assertEquals([
      Cookie::create('cookie1', 'foo', $expire, '/', '.example.com', $secure),
      Cookie::create('cookie2', 'bar', $expire, '/', '.example.com', $secure),
      // This might cause problems with Symfony 5?
      new Cookie('cookie3', NULL, 1, '/', '.example.com', $secure, TRUE, FALSE, NULL),
      new Cookie('cookie3', NULL, 1, '/', '', $secure, TRUE, FALSE, NULL),
    ], $response->headers->getCookies());
  }

}
